@@ -210,15 +210,8 @@ end
 -- @param id The data.history identifier
 local function history_save(id)
     if data.history[id] then
-        local f = io.open(id, "w")
-        if not f then
-            local i = 0
-            for d in id:gmatch(".-/") do
-                i = i + #d
-            end
-            gfs.mkdir(id:sub(1, i - 1))
-            f = assert(io.open(id, "w"))
-        end
+        assert(gfs.make_parent_directories(id))
+        local f = assert(io.open(id, "w"))
         for i = 1, math.min(#data.history[id].table, data.history[id].max) do
             f:write(data.history[id].table[i] .. "\n")
         end
@@ -315,7 +308,7 @@ end
 -- @callback exe_callback
 -- @tparam string command The command (as entered).
 
---- The callback function to call to get completion.
+--- The callback function to get completions.
 -- @usage local function my_completion_cb(command_before_comp, cur_pos_before_comp, ncomp)
 --    return command_before_comp.."foo", cur_pos_before_comp+3, 1
 -- end
@@ -323,7 +316,7 @@ end
 -- @callback completion_callback
 -- @tparam string command_before_comp The current command.
 -- @tparam number cur_pos_before_comp The current cursor position.
--- @tparam number ncomp The number of completion elements.
+-- @tparam number ncomp The number of the currently completed element.
 -- @treturn string command
 -- @treturn number cur_pos
 -- @treturn number matches
@@ -501,6 +494,9 @@ function prompt.run(args, textbox, exe_callback, completion_callback,
     completion_callback = completion_callback or args.completion_callback
     exe_callback        = exe_callback        or args.exe_callback
     textbox             = textbox             or args.textbox
+    if not textbox then
+        return
+    end
 
     search_term=nil
 
@@ -510,9 +506,6 @@ function prompt.run(args, textbox, exe_callback, completion_callback,
     local cur_pos = (selectall and 1) or text:wlen() + 1
     -- The completion element to use on completion request.
     local ncomp = 1
-    if not textbox then
-        return
-    end
 
     -- Build the hook map
     for _,v in ipairs(args.hooks or {}) do
@@ -562,7 +555,6 @@ function prompt.run(args, textbox, exe_callback, completion_callback,
             if args.keyreleased_callback then
                 args.keyreleased_callback(mod, key, command)
             end
-
             return
         end
 
@@ -797,7 +789,7 @@ function prompt.run(args, textbox, exe_callback, completion_callback,
         else
             if completion_callback then
                 if key == "Tab" or key == "ISO_Left_Tab" then
-                    if key == "ISO_Left_Tab" then
+                    if key == "ISO_Left_Tab" or mod.Shift then
                         if ncomp == 1 then return end
                         if ncomp == 2 then
                             command = command_before_comp
@@ -806,6 +798,8 @@ function prompt.run(args, textbox, exe_callback, completion_callback,
                                 text = command_before_comp, text_color = inv_col, cursor_color = cur_col,
                                 cursor_pos = cur_pos, cursor_ul = cur_ul, selectall = selectall,
                                 prompt = prettyprompt })
+                            cur_pos = cur_pos_before_comp
+                            ncomp = 1
                             return
                         end
 
@@ -823,7 +817,7 @@ function prompt.run(args, textbox, exe_callback, completion_callback,
                         exec(exe_callback)
                         return
                     end
-                else
+                elseif key ~= "Shift_L" and key ~= "Shift_R" then
                     ncomp = 1
                 end
             end
