@@ -1,24 +1,23 @@
 ---------------------------------------------------------------------------
+--- Remote control module allowing usage of awesome-client.
+--
 -- @author Julien Danjou &lt;julien@danjou.info&gt;
 -- @copyright 2009 Julien Danjou
--- @release v3.5.8
+-- @module awful.remote
 ---------------------------------------------------------------------------
 
 -- Grab environment we need
 require("awful.dbus")
-local load = loadstring or load -- v5.1 - loadstring, v5.2 - load
+local load = loadstring or load -- luacheck: globals loadstring (compatibility with Lua 5.1)
 local tostring = tostring
 local ipairs = ipairs
 local table = table
-local unpack = unpack or table.unpack -- v5.1: unpack, v5.2: table.unpack
+local unpack = unpack or table.unpack -- luacheck: globals unpack (compatibility with Lua 5.1)
 local dbus = dbus
 local type = type
 
---- Remote control module allowing usage of awesome-client.
--- awful.remote
-
 if dbus then
-    dbus.connect_signal("org.naquadah.awesome.awful.Remote", function(data, code)
+    dbus.connect_signal("org.awesomewm.awful.Remote", function(data, code)
         if data.member == "Eval" then
             local f, e = load(code)
             if f then
