@@ -240,11 +240,15 @@ end
 -- @return A table with all created tags.
 function tag.new(names, screen, layout)
     screen = get_screen(screen or 1)
+    -- True if `layout` should be used as the layout of each created tag
+    local have_single_layout = (not layout) or (layout.arrange and layout.name)
     local tags = {}
     for id, name in ipairs(names) do
-        table.insert(tags, id, tag.add(name, {screen = screen,
-                                            layout = (layout and layout[id]) or
-                                                        layout}))
+        local l = layout
+        if not have_single_layout then
+            l = layout[id] or layout[1]
+        end
+        table.insert(tags, id, tag.add(name, {screen = screen, layout = l}))
         -- Select the first tag.
         if id == 1 then
             tags[id].selected = true
@@ -451,11 +455,20 @@ function tag.gettags(s)
     return s and s.tags or {}
 end
 
---- Find a tag by name
--- @tparam[opt] screen s The screen of the tag
+--- Find a tag by name.
+-- @tparam screen s The screen of the tag
 -- @tparam string name The name of the tag
 -- @return The tag found, or `nil`
+-- @usage -- For the current screen
+-- local t = awful.tag.find_by_name(awful.screen.focused(), "name")
+--
+-- -- For a screen index
+-- local t = awful.tag.find_by_name(screen[1], "name")
+--
+-- -- For all screens
+-- local t = awful.tag.find_by_name(nil, "name")
 function tag.find_by_name(s, name)
+    --TODO v5: swap the arguments and make screen [opt]
     local tags = s and s.tags or root.tags()
     for _, t in ipairs(tags) do
         if name == t.name then
@@ -1454,8 +1467,29 @@ capi.tag.connect_signal("request::select", tag.object.view_only)
 -- @signal property::urgent_count
 -- @see client.urgent
 
+--- Emitted when a screen is removed.
+-- This can be used to salvage existing tags by moving them to a new
+-- screen (or creating a virtual screen). By default, there is no
+-- handler for this request. The tags will be deleted. To prevent
+-- this, an handler for this request must simply set a new screen
+-- for the tag.
+-- @signal request::screen
+
+--- Emitted after `request::screen` if no new screen has been set.
+-- The tag will be deleted, this is a last chance to move its clients
+-- before they are sent to a fallback tag. Connect to `request::screen`
+-- if you wish to salvage the tag.
+-- @signal removal-pending
+
 capi.screen.connect_signal("tag::history::update", tag.history.update)
 
+-- Make sure the history is set early
+timer.delayed_call(function()
+    for s in capi.screen do
+        s:emit_signal("tag::history::update")
+    end
+end)
+
 capi.screen.connect_signal("removed", function(s)
     -- First give other code a chance to move the tag to another screen
     for _, t in pairs(s.tags) do
