@@ -33,11 +33,17 @@ util.table = {}
 --- The default shell used when spawing processes.
 util.shell = os.getenv("SHELL") or "/bin/sh"
 
+--- Execute a system command and road the output.
+-- This function implementation **has been removed** and no longer
+-- do anything. Use `awful.spawn.easy_async`.
+-- @deprecated awful.util.pread
+
 --- Display a deprecation notice, but only once per traceback.
 -- @deprecated deprecate
 -- @param[opt] see The message to a new method / function to use.
 -- @tparam table args Extra arguments
--- @tparam boolean args.raw Print the message as-is without the automatic context
+-- @tparam boolean args.raw Print the message as-is without the automatic
+--   context (but only append a leading dot).
 -- @tparam integer args.deprecated_in Print the message only when Awesome's
 --   version is equal to or greater than deprecated_in.
 -- @see gears.debug
@@ -91,9 +97,9 @@ end
 -- @return mkdir return code
 -- @see gears.filesystem
 function util.mkdir(dir)
-    util.deprecate("gears.filesystem.mkdir", {deprecated_in=5})
+    util.deprecate("gears.filesystem.make_directories", {deprecated_in=5})
 
-    return gfs.mkdir(dir)
+    return gfs.make_directories(dir)
 end
 
 --- Eval Lua code.
